import collections
import json
import logging
import os.path
from typing import Type

from pydantic import BaseModel

config_file = 'data/config.json'


def update(d, u):
    for k, v in u.items():
        if isinstance(v, collections.abc.Mapping):
            d[k] = update(d.get(k, {}), v)
        else:
            d[k] = v
    return d


if not os.path.exists('data'):
    os.mkdir('data')


class BotSettings(BaseModel):
    token: str
    owners: list[int]
    mongo: str
    debug: bool
    database: str


class BasicBotConfig(BaseModel):
    bot: BotSettings


def load_config(config_type: Type[BasicBotConfig], default_values: dict[str, ...]):
    config_values = {
        'bot': {
            'token': '',
            'owners': [],
            'mongo': '',
            'debug': False,
            'database': ''
        }
    }
    config_values = update(config_values, default_values)
    if os.path.exists(config_file):
        with open(config_file, 'r', encoding='utf-8') as f:
            user_config = json.load(f)
            config_values = update(config_values, user_config)
        with open(config_file, 'w', encoding='utf-8') as f:
            json.dump(config_values, f, ensure_ascii=True, sort_keys=True, indent=4)
    else:
        logging.error('Config was created, restart needed')
        exit(0)
    return config_type(**config_values)



from aiogram import types, F, Router
from aiogram.fsm.context import FSMContext
from aiogram.types import InlineKeyboardMarkup
from aiogram.utils.keyboard import InlineKeyboardBuilder

state_clear_router = Router()


def get_state_clear_markup() -> InlineKeyboardMarkup:
    builder = InlineKeyboardBuilder()
    builder.button(text='❌ Отмена', callback_data='state_clear')
    builder.adjust(1, repeat=True)
    return builder.as_markup()


@state_clear_router.callback_query(F.data == 'state_clear')
async def on_state_clear_callback(query: types.CallbackQuery, state: FSMContext):
    await query.answer('✅ Действие отменено')
    await query.message.delete()
    await state.clear()



from datetime import datetime
from typing import Union

from aiogram.types import Message, CallbackQuery
from aiogram.utils.keyboard import InlineKeyboardBuilder


def get_timestamp(time: int) -> str:
    return datetime.fromtimestamp(time + 3600 * 3).strftime("%d.%m.%Y %H:%M")


async def confirm_action(data: Union[Message, CallbackQuery], description: str, warning: bool,
                         callback_data: str):
    builder = InlineKeyboardBuilder()

    builder.button(text='✅ Подтвердить', callback_data=callback_data)
    builder.button(text='❌ Отмена', callback_data='state_clear')

    if type(data) is Message:
        func = data.answer
    else:
        func = data.message.edit_text

    await func(
        f'Вы уверены, что хотите {description}?' + ('\n\n⚠️ Это действие необратимо' if warning else ''),
        reply_markup=builder.as_markup())



from aiogram import Dispatcher, Router, Bot
from aiogram.enums import ParseMode


class XBot:
    def __init__(self, token: str):
        self.router = Router()

        self.dispatcher = Dispatcher()
        self.dispatcher.include_router(self.router)

        self.main_bot = Bot(token, parse_mode=ParseMode.HTML)

    async def start(self):
        await self.dispatcher.start_polling(self.main_bot)



import random
import string
from typing import Optional

from aiogram import Bot, Dispatcher, Router
from aiogram.client.session.aiohttp import AiohttpSession
from aiogram.enums import ParseMode
from aiogram.exceptions import TelegramUnauthorizedError
from aiogram.types import User
from aiogram.webhook.aiohttp_server import SimpleRequestHandler, TokenBasedRequestHandler, setup_application
from aiohttp import web

MAIN_BOT_PATH = "/webhook/main"
OTHER_BOTS_PATH = "/webhook/bot/{bot_token}"


class XMultiBot:
    def __init__(self, main_token: str, base_url: str, port: int, startup_tokens: list[str]):
        self.router = Router()

        self._dispatcher = Dispatcher()
        self._dispatcher.include_router(self.router)

        self.minions: dict[str, Bot] = {}

        self._bot_settings = {"session": AiohttpSession(), "parse_mode": ParseMode.HTML}
        self.main_bot = Bot(main_token, **self._bot_settings)

        self._secret = ''.join([random.choice(string.ascii_letters + string.digits) for _ in range(64)])
        self._port = port
        self._base_url = base_url

        async def on_startup(bot: Bot):
            for token in startup_tokens:
                new_bot = Bot(token, self.main_bot.session)
                try:
                    await new_bot.get_me()
                except TelegramUnauthorizedError:
                    continue
                await new_bot.delete_webhook(drop_pending_updates=True)
                await new_bot.set_webhook("/webhook/bot/" + token)
                self.minions[token] = new_bot

            await bot.set_webhook(f"{self._base_url}{MAIN_BOT_PATH}", secret_token=self._secret)

        self._dispatcher.startup.register(on_startup)

    async def add_minion(self, token: str) -> Optional[User]:
        new_bot = Bot(token, **self._bot_settings)
        try:
            bot_user = await new_bot.get_me()
        except TelegramUnauthorizedError:
            return None
        await new_bot.delete_webhook(drop_pending_updates=True)
        await new_bot.set_webhook(f'{self._base_url}{OTHER_BOTS_PATH.format(bot_token=token)}')
        self.minions[token] = new_bot

        return bot_user

    async def delete_minion(self, token: str) -> bool:
        if token in self.minions:
            await self.minions[token].delete_webhook()
            del self.minions[token]
            return True
        else:
            return False

    async def start(self):
        app = web.Application()
        SimpleRequestHandler(dispatcher=self._dispatcher, bot=self.main_bot,
                             secret_token=self._secret).register(app,
                                                                 path=MAIN_BOT_PATH)
        TokenBasedRequestHandler(
            dispatcher=self._dispatcher,
            bot_settings=self._bot_settings
        ).register(app, path=OTHER_BOTS_PATH)

        setup_application(app, self._dispatcher, bot=self.main_bot)

        web.run_app(app, host='0.0.0.0', port=self._port)
